"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const visitor_plugin_common_1 = require("@graphql-codegen/visitor-plugin-common");
const angular_selection_set_to_object_1 = require("./angular-selection-set-to-object");
const angular_operation_variables_to_object_1 = require("./angular-operation-variables-to-object");
function getRootType(operation, schema) {
    switch (operation) {
        case 'query':
            return schema.getQueryType();
        case 'mutation':
            return schema.getMutationType();
        case 'subscription':
            return schema.getSubscriptionType();
    }
}
class AngularOperationsVisitor extends visitor_plugin_common_1.BaseDocumentsVisitor {
    constructor(schema, config, allFragments) {
        super(config, {
            avoidOptionals: config.avoidOptionals || false,
            immutableTypes: config.immutableTypes || false
        }, schema);
        this.setSelectionSetHandler(new angular_selection_set_to_object_1.AngularSelectionSetToObject(this.scalars, this.schema, this.convertName, this.config.addTypename, this.config.nonOptionalTypename, allFragments, this.config));
        this.setVariablesTransformer(new angular_operation_variables_to_object_1.AngularOperationVariablesHelper(this.scalars, this.convertName, this.config.avoidOptionals, this.config.immutableTypes));
    }
    OperationDefinition(node) {
        const name = this.handleAnonymousOperation(node);
        let resultType = getRootType(node.operation, this._schema);
        let resultSelections = node.selectionSet;
        /*
         If the operation has only one field in the result set, skip the name of the operation from type
         For instance if the an operation is of
         mutation CreatePost($input: CreatePostInput!) {
        createPost(input: $input) {
          __typename
          id
          title
        }
      }
    
      we want the method to just return CreatePostMutation of shape
      export type CreatePostMutation = { __typename: "Post" } &
        Pick<Post, "id" | "title">;
    
      and not include the name of the operation createPost
      export type CreatePostMutation = { __typename?: "Mutation" } & {
        createPost: Maybe<
          { __typename: "Post" } & Pick<Post, "id" | "title">
        >;
      };
         */
        if (resultSelections.selections && resultSelections.selections.length === 1) {
            resultSelections = node.selectionSet.selections[0];
            if (resultSelections.kind === 'InlineFragment') {
                resultType = visitor_plugin_common_1.getBaseType(resultType.getFields()[resultSelections.typeCondition.name.value]
                    .type);
            }
            else {
                resultType = visitor_plugin_common_1.getBaseType(resultType.getFields()[resultSelections.name.value].type);
            }
        }
        if (!resultType) {
            throw new Error(`Unable to find root schema type for operation type "${node.operation}"!`);
        }
        const selectionSet = this._selectionSetToObject.createNext(resultType, resultSelections.selectionSet);
        const operationResult = new visitor_plugin_common_1.DeclarationBlock(this._declarationBlockConfig)
            .export()
            .asKind('type')
            .withName(this.convertName(name, {
            suffix: visitor_plugin_common_1.toPascalCase(node.operation)
        }))
            .withContent(selectionSet.string).string;
        return operationResult;
    }
    handleAnonymousOperation(node) {
        const name = node.name && node.name.value;
        if (name) {
            return this.convertName(node, {
                useTypesPrefix: false
            });
        }
        return this.convertName(this._unnamedCounter++ + '', {
            prefix: 'Unnamed_',
            suffix: '_',
            useTypesPrefix: false
        });
    }
}
exports.AngularOperationsVisitor = AngularOperationsVisitor;
//# sourceMappingURL=angular-operation-vistor.js.map